% Training function for SVM
%
% Usage:
%     scratch = train_svm(trainpats, trainregs, args);
%
% Input parameters:
%     trainpats: training patterns
%     trainregs: training class regressors
%     in_args: further arguments
%
% Return values:
%     scratch: contains the model that the classifier has learned
%
% -------------------------------------------------------------------------
% This is a support vector machine training function. It train the
% classifier and makes is ready for testing. 
% NOTE : This classfier currently does only two category classification
% only. We have to implement the multi-class classification.
% Make sure you pass in only two regressors  
%
% You need to call TEST_SVM afterwards to assess how well this
% generalizes to the test data.
%
% This function internally uses the svm library generated by the
% Joachims (SVM-light). This function is a wrapper script that converts the
% subj struture into the form required by this library.

% PATS = nFeatures x nTimepoints
% TARGS = nOuts x nTimepoints
%
% The classifier functions use a IN_ARGS structure to store possible
% arguments (rather than a varargin and property/value pairs). This
% tends to be easier to manage when lots of arguments are
% involved.
%
% IN_ARGS are the various arguments that can be passed in for type
% of kernels used and the learning parameters.
%
% IN_ARGS: 
%  1] in_args.kernel_type = 0 'LINEAR'
%  2] in_args.kernel_type = 1 'POLYNOMIAL'
%  3] in_args.kernel_type = 2 'RBF (Radial Basis Function)'
%  4] in_args.kernel_type = 4 'SIGMOID'
%
% Adapted from the MVPA toolbox.
% -------------------------------------------------------------------------
function scratch = train_svm(trainpats, trainregs, args)
    
    % Process input arguments
    defaults.kernelString = '';
    args = mergestructs(args, defaults);
    
    % Check inputs
    args = sanity_check(trainpats, trainregs, args);
    
    % SETTING THINGS UP
    % This classifier needs the two caterogies to be in the form of +/- 1 
    % in this step we will convert catergory 1 into all the 1 and
    % category 2 into -1
    [train_max_val train_max_idx]  = max(trainregs);
    train_max_idx(train_max_idx == 1) = 1;
    train_max_idx(train_max_idx == 2) = -1;
    
    % TRAINING THE CLASSIFIER
    clear fun mexsvmclassify;
    clear fun mexsvmlearn;
    model = mexsvmlearn(trainpats', train_max_idx', args.kernelString);
    clear fun mexsvmlearn;
    
    % Return model in scratch
    scratch.model = model;
end

% -------------------------------------------------------------------------
function args = sanity_check(trainpats, trainregs, args)

    if size(trainpats,2) ~= size(trainregs,2)
        error('Different number of training pats and regs timepoints');
    end
    if size(trainregs,1) ~= 2
        error('Cannot classify more than two categories');
    end
    
    [isbool isrest isoveractive] = check_1ofn_regressors(trainregs);
    if ~isbool || isrest || isoveractive
        if ~args.ignore_1ofn
            warning('Not 1-of-n regressors');
        end
    end
end
